"""
Name: EK_AF_UCS_Main.py
Authors: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Feb. 12, 2010
Description:    Main script used to call EK_AF_UCS - handles arguments with option_parser, sets and stores all global parameters, and calls 
                 for the environment's creating, and for instantiation of the algorithm.
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 1.0: An learning classifier system algorithm implemented with our new attribute tracking, feedback, and expert knowledge strategies for
modeling epistasis and heterogeneity data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python
#Import modules
from EK_AF_UCS_Algorithm import *
from EK_AF_UCS_Environment import *
from EK_AF_UCS_Constants import *
import time
import sys
from optparse import OptionParser

# Argument/Option Parser ---------------------------------------------------------------------------------------------
usage = "usage: %prog [options] please see help"  #NEEDS TO BE FLESHED OUT! #include reference to how to use OptionParser and example calls.

parser = OptionParser(usage)

#No Default options - filenames/paths
parser.add_option("--lfile", action="store", type="string", dest="trainFile", help="path name of text file upon which algorithm will train. [DEFAULT: None]")
parser.add_option("--tfile", action="store", type="string", dest="testFile", help="path name of text file upon which algorithm will be tested. [DEFAULT: None]")
parser.add_option("--ofile", action="store", type="string", dest="outFile", help="path name basis for all output files generated by algorithm. [DEFAULT: None]")#MIGHT REQUIRE DEFAULT//

#Standard Run Options
parser.add_option("--env", action="store", dest="environment", help="what is the data set/attribute style... this dictates rule representation [DEFAULT: %default]", default = "SNP")
parser.add_option("--cv", action="store_false", dest="crossVal", help="is cross validation used? (i.e. is there a test file) [DEFAULT: %default] ", default = True)
parser.add_option("--track", action="store", type="int", dest="trackFreq", help="number of learning iterations between basic performance reporting. [DEFAULT: %default --> tracking occurs at train-data-sized intervals]", default = 0)
parser.add_option("--li", action="store", type="string", dest="learnIterations", help="string of (int) learning iterations where last value is the total iterations to run, and each preceding value indicates a complete evaluation point. Each value separated by period. [DEFAULT: %default]", default = '100000.500000.1000000')
parser.add_option("--pop", action="store", type="int", dest="popSize", help="the maximum size of the evolving population [DEFAULT: %default]", default = 1000)

#Standard Run Constants
#parser.add_option("--reward", action="store", type="int", dest="reward", help="learning parameter determining magnitude of reward. [DEFAULT: %default]", default = 1000)
parser.add_option("--beta", action="store", type="float", dest="beta", help="The learning rate for updating fitness [DEFAULT: %default]", default = 0.2)
parser.add_option("--thetaG", action="store", type="int", dest="theta_GA", help="GA activation experience threshold [DEFAULT: %default]", default = 25)
parser.add_option("--pX", action="store", type="float", dest="pX", help="The probability of applying crossover in an offspring classifier. [DEFAULT: %default]", default = 0.8)
parser.add_option("--fitRed", action="store", type="float", dest="fitRed", help="The reduction in fitness received when GA is employed [DEFAULT: %default]", default = 0.1)
parser.add_option("--thetaSel", action="store", type="float", dest="theta_Sel", help="The fraction of the action set to be included in tournament selection [DEFAULT: %default]", default = 0.5)
parser.add_option("--wi", action="store", type="string", dest="wild", help=" The symbol used to represent a 'wild card' [DEFAULT: %default", default = "#") #tournament, roulete
parser.add_option("--accIn", action="store", type="float", dest="accuracyIni", help="Initial accuracy of new classifier. [DEFAULT: %default]", default = 0.0)
parser.add_option("--fitIn", action="store", type="float", dest="fitnessIni", help="Initial fitness of new classifier. [DEFAULT: %default]", default = 0.01)
parser.add_option("--pnc", action="store", type="float", dest="p_nocare", help="Probability of adding wild to new classifier [DEFAULT: %default]", default = 0.5)
parser.add_option("--pM", action="store", type="float", dest="pMut", help="The probability of applying mutation within a given rule attribute. [DEFAULT: %default]", default = 0.04)
parser.add_option("--thetaSub", action="store", type="int", dest="theta_Sub", help="The experience of a classifier required to be a subsumer. [DEFAULT: %default]", default = 20)
parser.add_option("--accZ", action="store", type="float", dest="accZ", help="Subsumption accuracy requirement [DEFAULT: %default]", default = 0.99)
parser.add_option("--delta", action="store", type="float", dest="delta", help="The fraction of the mean fitness of the population below which the fitness of a classifier may be considered in its vote for deletion [DEFAULT: %default]", default = 0.1)
parser.add_option("--thetaDel", action="store", type="int", dest="theta_Del", help="Specified the threshold over which the fitness of a classifier may be considered in its deletion probability. [DEFAULT: %default]", default = 20)
parser.add_option("--fitPow", action="store", type="int", dest="fitnessPower", help="Exponential used to calculate fitness based on accuracy. [DEFAULT: %default]", default = 10)

#Variable Algorithm Mechanisms
parser.add_option("--sub", action="store", dest="subsumption", help="turn on subsumption (a generalization pressure) [DEFAULT: %default]", default = True) #Might not work
parser.add_option("--sel", action="store", type="string", dest="selection", help="specify the selection method [DEFAULT: %default", default = "tournament") #tournament, roulete
parser.add_option("--cover", action="store", type="string", dest="covering", help="specify the pop. initialization method [DEFAULT: %default", default = "smart") 
parser.add_option("--mut", action="store", dest="mutation", help="inactivates the mutation operator [DEFAULT: %default]", default = True) #MAY want to change to multiple mutation styles
parser.add_option("--cross", action="store", type="string", dest="crossover", help="type of crossover used [DEFAULT: %default]", default = "spc") # none, spc, dpc, huc, uc

#Expert Knowledge Source for Attributes
parser.add_option("--ek", action="store", dest="expertK", help="saves destination of expert knowlege for incorporation [DEFAULT: %default]", default = 'False')#source of EK file
parser.add_option("--ekmax",action="store", dest="EK_max", help = " percent over chance that maximum EK prob will hold [DEFAULT: %default]", default = 0.4)
parser.add_option("--ektype",action="store", dest="EK_type", help = "The manner in which the EK weight lists are scaled. Impacts how attributes are picked by GA [DEFAULT: %default]", default = 'simpleTransformation')#simpleTransformation, simpleLogit
parser.add_option("--ekapply",action="store", dest="EK_apply", help = "The manner in which the EK weight lists are scaled. Impacts how attributes are picked by GA [DEFAULT: %default]", default = 'cover')#cover, mutation, both
parser.add_option("--ekdop",action="store", dest="EK_dop", help = " digits of percision [DEFAULT: %default]", default = 5)

#Covering Mechanism
parser.add_option("--coverGen", action="store",dest="coverGenerality", help = "determines covering generality [DEFAULT: %default]",default = 0.5) #Doubles as DCP for logistic EK transformation.

#Local Predictive Rule Search
parser.add_option("--predT", action="store_true", dest="p_rule_search", help="novel predictive rule searching operator [DEFAULT: %default]", default = False)
parser.add_option("--predF", action="store_false", dest="p_rule_search")

#Fitness Calculation
parser.add_option("--fitCalc", action="store", type="string", dest="fitcalc", help="novel predictive rule searching operator [DEFAULT: %default]", default = "accuracy")

#Delete Default rules
parser.add_option("--drulesT", action="store_true", dest="defaultRuleDelete", help="periodically removes default rules - *before complete evaluations [DEFAULT: %default]", default = False)
parser.add_option("--drulesF", action="store_false", dest="defaultRuleDelete")

#Evolved Population PickUp
parser.add_option("--restart", action="store", dest="remakeFile", help="saves path and most of file name of previously evolved population for restart [DEFAULT: %default]", default = 'False') #or filename given
#parser.add_option("--restartF", action="store", dest="remakeFile", help="saves path and most of file name of previously evolved population for restart [DEFAULT: %default]", default = False) #or filename given

#AttributeDiscoveryMetric
parser.add_option("--adm",action="store",dest="attDisc",help = "Determines how power is calculated, what internal knowledge is used for learning feedback [DEFAULT: %default]",default = 'Spec') #Spec, FWSpec, AWSpec
parser.add_option("--adivT",action="store_true",dest="attDiv",help = " [DEFAULT: %default]",default = False) 
parser.add_option("--adivF",action="store_false",dest="attDiv") 

#Attribute Tracking
parser.add_option("--attTrackT",action="store_true",dest="attributeTracking",help = "Is attribute tracking on?  This tracks which attributes were important for making accurate predictions about a given data instance. [DEFAULT: %default]",default = False)
parser.add_option("--attTrackF",action="store_false",dest="attributeTracking")
parser.add_option("--attFeedT",action="store_true",dest="attributeFeedback",help = "Is attribute tracking on?  This tracks which attributes were important for making accurate predictions about a given data instance. [DEFAULT: %default]",default = False)
parser.add_option("--attFeedF",action="store_false",dest="attributeFeedback")
parser.add_option("--attPunT",action="store_true",dest="attributePunish",help = "Is attribute tracking on?  This tracks which attributes were important for making accurate predictions about a given data instance. [DEFAULT: %default]",default = False)
parser.add_option("--attPunF",action="store_false",dest="attributePunish")
parser.add_option("--attIDT",action="store_true",dest="attributeID",help = "Is attribute tracking on?  This tracks which attributes were important for making accurate predictions about a given data instance. [DEFAULT: %default]",default = False)
parser.add_option("--attIDF",action="store_false",dest="attributeID")

(options, args) = parser.parse_args()

#Pass constants to EK_AF_UCS_Constants class
cons.setRunOps(options.popSize, options.crossVal, options.trackFreq, options.trainFile, options.testFile, options.environment, options.learnIterations, options.remakeFile, options.expertK, options.defaultRuleDelete)
cons.setConstants(options.beta, options.theta_GA, options.pX, options.fitRed, options.theta_Sel, options.wild, options.accuracyIni, options.fitnessIni, options.p_nocare, options.pMut, options.theta_Sub, options.accZ, options.delta, options.theta_Del, options.fitnessPower)
cons.setMech(options.subsumption, options.selection, options.covering, options.mutation, options.crossover, options.EK_type, options.p_rule_search, options.fitcalc, options.coverGenerality, options.attDisc, options.attributeTracking, options.attributeFeedback, options.EK_apply, options.attDiv, options.attributePunish, options.attributeID, options.EK_dop)

#Engage Timer
timer = Timer()
    
#Establish Environment
if options.environment == 'SNP':
    e = Environment()
    e = SNP_Environment(options.trainFile, options.testFile, options.remakeFile)
else:
    print "Algorithm is not set up to handle requested environment."
    
#Instantiate and Run EK_AF_UCS
ekafucs = EK_AF_UCS(e, options.outFile, options.learnIterations, timer, options.remakeFile) # arguments include the environment, output file's root name, and the unparsed learning iterations

cons.refEnv(e)
if options.expertK != 'False':  #If there is an expert knowledge filename
    timer.startTimeExpertInit()
    cons.makeExpert(options.EK_max)
    timer.stopTimeExpertInit()
    
ekafucs.runEK_AF_UCS() 
